@@ -37,6 +37,7 @@ GFXRECON_BEGIN_NAMESPACE(decode)
 
 struct ApiCallInfo
 {
+    uint64_t         index{ 0 };
     format::ThreadId thread_id{ 0 };
 };
 
@@ -45,6 +46,8 @@ class ApiDecoder
   public:
     virtual ~ApiDecoder() {}
 
+    virtual void WaitIdle() = 0;
+
     virtual bool SupportsApiCall(format::ApiCallId id) = 0;
 
     virtual bool SupportsMetaDataId(format::MetaDataId meta_data_id) = 0;
