@@ -37,7 +37,7 @@ GFXRECON_BEGIN_NAMESPACE(decode)
 
 FileProcessor::FileProcessor() :
     file_header_{}, file_descriptor_(nullptr), current_frame_number_(0), bytes_read_(0),
-    error_state_(kErrorInvalidFileDescriptor), annotation_handler_(nullptr), compressor_(nullptr)
+    error_state_(kErrorInvalidFileDescriptor), annotation_handler_(nullptr), compressor_(nullptr), api_call_index_(0)
 {}
 
 FileProcessor::~FileProcessor()
@@ -56,6 +56,14 @@ FileProcessor::~FileProcessor()
     DecodeAllocator::DestroyInstance();
 }
 
+void FileProcessor::WaitDecodersIdle()
+{
+    for (auto decoder : decoders_)
+    {
+        decoder->WaitIdle();
+    }
+};
+
 bool FileProcessor::Initialize(const std::string& filename)
 {
     bool success = false;
@@ -157,7 +165,7 @@ bool FileProcessor::ProcessFileHeader()
 
                 if ((compressor_ == nullptr) && (enabled_options_.compression_type != format::CompressionType::kNone))
                 {
-                    GFXRECON_LOG_ERROR("Failed to initialized file compression module (type = %u); replay of "
+                    GFXRECON_LOG_ERROR("Failed to initialize file compression module (type = %u); replay of "
                                        "compressed data will not be possible",
                                        enabled_options_.compression_type);
                     success      = false;
@@ -388,8 +396,8 @@ bool FileProcessor::ProcessFunctionCall(const format::BlockHeader& block_header,
     size_t      parameter_buffer_size = static_cast<size_t>(block_header.size) - sizeof(call_id);
     uint64_t    uncompressed_size     = 0;
     ApiCallInfo call_info             = {};
-
-    bool success = ReadBytes(&call_info.thread_id, sizeof(call_info.thread_id));
+    call_info.index                   = api_call_index_;
+    bool success                      = ReadBytes(&call_info.thread_id, sizeof(call_info.thread_id));
 
     if (success)
     {
@@ -446,6 +454,8 @@ bool FileProcessor::ProcessFunctionCall(const format::BlockHeader& block_header,
                     DecodeAllocator::End();
                 }
             }
+
+            ++api_call_index_;
         }
     }
     else
@@ -875,7 +885,7 @@ bool FileProcessor::ProcessMetaData(const format::BlockHeader& block_header, for
         success = success && ReadBytes(&header.data_size, sizeof(header.data_size));
 
         // Read variable size shader group handle data into parameter_buffer_.
-        success = success && ReadParameterBuffer(header.data_size);
+        success = success && ReadParameterBuffer(static_cast<size_t>(header.data_size));
 
         if (success)
         {
@@ -886,7 +896,7 @@ bool FileProcessor::ProcessMetaData(const format::BlockHeader& block_header, for
                     decoder->DispatchSetRayTracingShaderGroupHandlesCommand(header.thread_id,
                                                                             header.device_id,
                                                                             header.pipeline_id,
-                                                                            header.data_size,
+                                                                            static_cast<size_t>(header.data_size),
                                                                             parameter_buffer_.data());
                 }
             }
@@ -1195,7 +1205,6 @@ bool FileProcessor::ProcessAnnotation(const format::BlockHeader& block_header, f
 
     success = ReadBytes(&label_length, sizeof(label_length));
     success = success && ReadBytes(&data_length, sizeof(data_length));
-
     if (success)
     {
         if ((label_length > 0) || (data_length > 0))
