@@ -76,6 +76,8 @@ class VulkanReplayConsumerBase : public VulkanConsumer
 
     void SetFpsInfo(graphics::FpsInfo* fps_info) { fps_info_ = fps_info; }
 
+    virtual void WaitDevicesIdle() override;
+
     virtual void ProcessStateBeginMarker(uint64_t frame_number) override;
 
     virtual void ProcessStateEndMarker(uint64_t frame_number) override;
@@ -152,18 +154,21 @@ class VulkanReplayConsumerBase : public VulkanConsumer
                                          const std::vector<uint64_t>& level_sizes,
                                          const uint8_t*               data) override;
 
-    virtual void Process_vkUpdateDescriptorSetWithTemplate(format::HandleId                 device,
+    virtual void Process_vkUpdateDescriptorSetWithTemplate(const ApiCallInfo&               call_info,
+                                                           format::HandleId                 device,
                                                            format::HandleId                 descriptorSet,
                                                            format::HandleId                 descriptorUpdateTemplate,
                                                            DescriptorUpdateTemplateDecoder* pData) override;
 
-    virtual void Process_vkCmdPushDescriptorSetWithTemplateKHR(format::HandleId commandBuffer,
-                                                               format::HandleId descriptorUpdateTemplate,
-                                                               format::HandleId layout,
-                                                               uint32_t         set,
+    virtual void Process_vkCmdPushDescriptorSetWithTemplateKHR(const ApiCallInfo& call_info,
+                                                               format::HandleId   commandBuffer,
+                                                               format::HandleId   descriptorUpdateTemplate,
+                                                               format::HandleId   layout,
+                                                               uint32_t           set,
                                                                DescriptorUpdateTemplateDecoder* pData) override;
 
-    virtual void Process_vkUpdateDescriptorSetWithTemplateKHR(format::HandleId                 device,
+    virtual void Process_vkUpdateDescriptorSetWithTemplateKHR(const ApiCallInfo&               call_info,
+                                                              format::HandleId                 device,
                                                               format::HandleId                 descriptorSet,
                                                               format::HandleId                 descriptorUpdateTemplate,
                                                               DescriptorUpdateTemplateDecoder* pData) override;
@@ -501,6 +506,13 @@ class VulkanReplayConsumerBase : public VulkanConsumer
                                  const StructPointerDecoder<Decoded_VkSubmitInfo>* pSubmits,
                                  const FenceInfo*                                  fence_info);
 
+    VkResult OverrideQueueSubmit2(PFN_vkQueueSubmit2                                 func,
+                                  VkResult                                           original_result,
+                                  const QueueInfo*                                   queue_info,
+                                  uint32_t                                           submitCount,
+                                  const StructPointerDecoder<Decoded_VkSubmitInfo2>* pSubmits,
+                                  const FenceInfo*                                   fence_info);
+
     VkResult OverrideQueueBindSparse(PFN_vkQueueBindSparse                                 func,
                                      VkResult                                              original_result,
                                      const QueueInfo*                                      queue_info,
@@ -857,6 +869,12 @@ class VulkanReplayConsumerBase : public VulkanConsumer
                                                         uint32_t                                 groupCount,
                                                         size_t                                   dataSize,
                                                         PointerDecoder<uint8_t>*                 pData);
+    VkResult OverrideGetAndroidHardwareBufferPropertiesANDROID(
+        PFN_vkGetAndroidHardwareBufferPropertiesANDROID                         func,
+        VkResult                                                                original_result,
+        const DeviceInfo*                                                       device_info,
+        const struct AHardwareBuffer*                                           hardware_buffer,
+        StructPointerDecoder<Decoded_VkAndroidHardwareBufferPropertiesANDROID>* pProperties);
 
   private:
     void RaiseFatalError(const char* message) const;
@@ -919,15 +937,27 @@ class VulkanReplayConsumerBase : public VulkanConsumer
     void GetImportedSemaphores(const HandlePointerDecoder<VkSemaphore>& semaphore_data,
                                std::vector<const SemaphoreInfo*>*       imported_semaphores);
 
+    void GetImportedSemaphores(const StructPointerDecoder<Decoded_VkSemaphoreSubmitInfo>* semaphore_info_data,
+                               std::vector<const SemaphoreInfo*>*                         imported_semaphores);
+
     void GetShadowSemaphores(const HandlePointerDecoder<VkSemaphore>& semaphore_data,
                              std::vector<const SemaphoreInfo*>*       shadow_semaphores);
 
+    void GetShadowSemaphores(const StructPointerDecoder<Decoded_VkSemaphoreSubmitInfo>* semaphore_info_data,
+                             std::vector<const SemaphoreInfo*>*                         shadow_semaphores);
+
     void TrackSemaphoreForwardProgress(const HandlePointerDecoder<VkSemaphore>& semaphore_data,
                                        std::vector<const SemaphoreInfo*>*       removed_semaphores);
 
+    void TrackSemaphoreForwardProgress(const StructPointerDecoder<Decoded_VkSemaphoreSubmitInfo>* semaphore_info_data,
+                                       std::vector<const SemaphoreInfo*>*                         removed_semaphores);
+
     void GetNonForwardProgress(const HandlePointerDecoder<VkSemaphore>& semaphore_data,
                                std::vector<const SemaphoreInfo*>*       non_forward_progress_semaphores);
 
+    void GetNonForwardProgress(const StructPointerDecoder<Decoded_VkSemaphoreSubmitInfo>* semaphore_info_data,
+                               std::vector<const SemaphoreInfo*>* non_forward_progress_semaphores);
+
     VkResult CreateSwapchainImage(const DeviceInfo*        device_info,
                                   const VkImageCreateInfo* image_create_info,
                                   VkImage*                 image,
@@ -942,7 +972,7 @@ class VulkanReplayConsumerBase : public VulkanConsumer
     // When processing swapchain image state for the trimming state setup, acquire an image, transition it to
     // the expected layout, and then call queue present if the image is not expected to be in the acquired state so that
     // no more than one image is acquired at a time.
-    void ProcessSetSwapchainImageStateQueueSubmit(VkDevice          device,
+    void ProcessSetSwapchainImageStateQueueSubmit(const DeviceInfo* device_info,
                                                   SwapchainKHRInfo* swapchain_info,
                                                   uint32_t          last_presented_image,
                                                   const std::vector<format::SwapchainImageStateInfo>& image_info);
