@@ -87,10 +87,10 @@ format::ThreadId CaptureManager::ThreadData::GetThreadId()
 CaptureManager::CaptureManager(format::ApiFamilyId api_family) :
     api_family_(api_family), force_file_flush_(false), timestamp_filename_(true),
     memory_tracking_mode_(CaptureSettings::MemoryTrackingMode::kPageGuard), page_guard_align_buffer_sizes_(false),
-    page_guard_track_ahb_memory_(false), page_guard_memory_mode_(kMemoryModeShadowInternal), trim_enabled_(false),
-    trim_current_range_(0), current_frame_(kFirstFrame), capture_mode_(kModeWrite), previous_hotkey_state_(false),
-    debug_layer_(false), debug_device_lost_(false), screenshot_prefix_(""), screenshots_enabled_(false),
-    global_frame_count_(0)
+    page_guard_track_ahb_memory_(false), page_guard_unblock_sigsegv(false),
+    page_guard_memory_mode_(kMemoryModeShadowInternal), trim_enabled_(false), trim_current_range_(0),
+    current_frame_(kFirstFrame), capture_mode_(kModeWrite), previous_hotkey_state_(false), debug_layer_(false),
+    debug_device_lost_(false), screenshot_prefix_(""), screenshots_enabled_(false), global_frame_count_(0)
 {}
 
 CaptureManager::~CaptureManager()
@@ -245,6 +245,7 @@ bool CaptureManager::Initialize(std::string base_filename, const CaptureSettings
     {
         page_guard_align_buffer_sizes_ = trace_settings.page_guard_align_buffer_sizes;
         page_guard_track_ahb_memory_   = trace_settings.page_guard_track_ahb_memory;
+        page_guard_unblock_sigsegv     = trace_settings.page_guard_unblock_sigsegv;
 
         bool use_external_memory = trace_settings.page_guard_external_memory;
 
@@ -312,11 +313,13 @@ bool CaptureManager::Initialize(std::string base_filename, const CaptureSettings
         else if (!trace_settings.trim_key.empty())
         {
             trim_key_ = trace_settings.trim_key;
+            trim_key_frames_ = trace_settings.trim_key_frames;
 
             // Enable state tracking when hotkey pressed
             if (IsTrimHotkeyPressed())
             {
                 capture_mode_ = kModeWriteAndTrack;
+                trim_key_first_frame_ = current_frame_;
 
                 success = CreateCaptureFile(util::filepath::InsertFilenamePostfix(base_filename_, "_trim_trigger"));
             }
@@ -346,7 +349,8 @@ bool CaptureManager::Initialize(std::string base_filename, const CaptureSettings
         {
             util::PageGuardManager::Create(trace_settings.page_guard_copy_on_map,
                                            trace_settings.page_guard_separate_read,
-                                           util::PageGuardManager::kDefaultEnableReadWriteSamePage);
+                                           util::PageGuardManager::kDefaultEnableReadWriteSamePage,
+                                           trace_settings.page_guard_unblock_sigsegv);
         }
 
         if ((capture_mode_ & kModeTrack) == kModeTrack)
@@ -490,7 +494,8 @@ void CaptureManager::CheckContinueCaptureForWriteMode()
             }
         }
     }
-    else if (IsTrimHotkeyPressed())
+    else if (IsTrimHotkeyPressed() ||
+             ((trim_key_frames_ > 0) && (current_frame_ >= (trim_key_first_frame_ + trim_key_frames_))))
     {
         // Stop recording and close file.
         DeactivateTrimming();
@@ -523,6 +528,8 @@ void CaptureManager::CheckStartCaptureForTrackMode()
         bool success = CreateCaptureFile(util::filepath::InsertFilenamePostfix(base_filename_, "_trim_trigger"));
         if (success)
         {
+
+            trim_key_first_frame_ = current_frame_;
             ActivateTrimming();
         }
         else
@@ -610,8 +617,6 @@ std::string CaptureManager::CreateTrimFilename(const std::string&
 
 bool CaptureManager::CreateCaptureFile(const std::string& base_filename)
 {
-    auto state_lock = AcquireUniqueStateLock();
-
     bool        success          = true;
     std::string capture_filename = base_filename;
 
@@ -638,8 +643,6 @@ bool CaptureManager::CreateCaptureFile(const std::string& base_filename)
 
 void CaptureManager::ActivateTrimming()
 {
-    auto state_lock = AcquireUniqueStateLock();
-
     capture_mode_ |= kModeWrite;
 
     auto thread_data = GetThreadData();
@@ -650,8 +653,6 @@ void CaptureManager::ActivateTrimming()
 
 void CaptureManager::DeactivateTrimming()
 {
-    auto state_lock = AcquireUniqueStateLock();
-
     capture_mode_ &= ~kModeWrite;
     file_stream_ = nullptr;
 }
