@@ -321,9 +321,9 @@ void VulkanCaptureManager::WriteSetDeviceMemoryPropertiesCommand(
         // populate thread_data's scratch_buffer_ then write to file.
         auto& scratch_buffer = thread_data->GetScratchBuffer();
         scratch_buffer.clear();
-        scratch_buffer.insert(scratch_buffer.end(),
-                              reinterpret_cast<uint8_t*>(&memory_properties_cmd),
-                              reinterpret_cast<uint8_t*>(&memory_properties_cmd) + sizeof(memory_properties_cmd));
+        std::copy(reinterpret_cast<uint8_t*>(&memory_properties_cmd),
+                  reinterpret_cast<uint8_t*>(&memory_properties_cmd) + sizeof(memory_properties_cmd),
+                  std::back_inserter(scratch_buffer));
 
         format::DeviceMemoryType type;
         for (uint32_t i = 0; i < memory_properties.memoryTypeCount; ++i)
@@ -693,6 +693,14 @@ VkResult VulkanCaptureManager::OverrideCreateDevice(VkPhysicalDevice
             // to ensure it is available.
             wrapper->physical_device = physical_device_wrapper;
         }
+
+        for (uint32_t q = 0; q < pCreateInfo_unwrapped->queueCreateInfoCount; ++q)
+        {
+            const VkDeviceQueueCreateInfo* queue_create_info = &pCreateInfo_unwrapped->pQueueCreateInfos[q];
+            assert(wrapper->queue_family_creation_flags.find(queue_create_info->queueFamilyIndex) ==
+                   wrapper->queue_family_creation_flags.end());
+            wrapper->queue_family_creation_flags[queue_create_info->queueFamilyIndex] = queue_create_info->flags;
+        }
     }
 
     // Restore modified property/feature create info values to the original application values
@@ -1233,109 +1241,175 @@ VulkanCaptureManager::FindAllocateMemoryExtensions(const VkMemoryAllocateInfo* a
     return import_ahb_info;
 }
 
-void VulkanCaptureManager::ProcessImportAndroidHardwareBuffer(VkDevice         device,
-                                                              VkDeviceMemory   memory,
-                                                              AHardwareBuffer* hardware_buffer)
+bool VulkanCaptureManager::ProcessReferenceToAndroidHardwareBuffer(VkDevice device, AHardwareBuffer* hardware_buffer)
 {
 #if defined(VK_USE_PLATFORM_ANDROID_KHR)
-    auto memory_wrapper = reinterpret_cast<DeviceMemoryWrapper*>(memory);
-    assert((memory_wrapper != nullptr) && (hardware_buffer != nullptr));
+    assert(hardware_buffer != nullptr);
+    auto     device_wrapper   = reinterpret_cast<DeviceWrapper*>(device);
+    VkDevice device_unwrapped = device_wrapper->handle;
+    auto     device_table     = GetDeviceTable(device);
 
     auto entry = hardware_buffers_.find(hardware_buffer);
-    if (entry != hardware_buffers_.end())
-    {
-        ++entry->second.reference_count;
-        memory_wrapper->hardware_buffer = hardware_buffer;
-    }
-    else
+    if (entry == hardware_buffers_.end())
     {
         // If this is the first device memory object to reference the hardware buffer, write a buffer creation
         // command to the capture file and setup memory tracking.
-        void* data   = nullptr;
-        int   result = -1;
 
         std::vector<format::HardwareBufferPlaneInfo> plane_info;
 
-        // The multi-plane functions are declared for API 26, but are only available to link with API 29.  So, this
-        // could be turned into a run-time check dependent on dlsym returning a valid pointer for
-        // AHardwareBuffer_lockPlanes.
-#if __ANDROID_API__ >= 29
-        AHardwareBuffer_Planes ahb_planes;
-        result =
-            AHardwareBuffer_lockPlanes(hardware_buffer, AHARDWAREBUFFER_USAGE_CPU_READ_OFTEN, -1, nullptr, &ahb_planes);
-        if (result == 0)
-        {
-            data = ahb_planes.planes[0].data;
+        AHardwareBuffer_Desc desc;
+        AHardwareBuffer_describe(hardware_buffer, &desc);
 
-            for (uint32_t i = 0; i < ahb_planes.planeCount; ++i)
+        if ((desc.usage & AHARDWAREBUFFER_USAGE_CPU_READ_MASK) != 0)
+        {
+
+            void* data   = nullptr;
+            int   result = -1;
+
+            // The multi-plane functions are declared for API 26, but are only available to link with API 29.  So, this
+            // could be turned into a run-time check dependent on dlsym returning a valid pointer for
+            // AHardwareBuffer_lockPlanes.
+#if __ANDROID_API__ >= 29
+            AHardwareBuffer_Planes ahb_planes;
+            result = AHardwareBuffer_lockPlanes(
+                hardware_buffer, AHARDWAREBUFFER_USAGE_CPU_READ_OFTEN, -1, nullptr, &ahb_planes);
+            if (result == 0)
             {
-                format::HardwareBufferPlaneInfo ahb_plane_info;
-                ahb_plane_info.offset =
-                    reinterpret_cast<uint8_t*>(ahb_planes.planes[i].data) - reinterpret_cast<uint8_t*>(data);
-                ahb_plane_info.pixel_stride = ahb_planes.planes[i].pixelStride;
-                ahb_plane_info.row_pitch    = ahb_planes.planes[i].rowStride;
-                plane_info.emplace_back(std::move(ahb_plane_info));
+                data = ahb_planes.planes[0].data;
+
+                for (uint32_t i = 0; i < ahb_planes.planeCount; ++i)
+                {
+                    format::HardwareBufferPlaneInfo ahb_plane_info;
+                    ahb_plane_info.offset =
+                        reinterpret_cast<uint8_t*>(ahb_planes.planes[i].data) - reinterpret_cast<uint8_t*>(data);
+                    ahb_plane_info.pixel_stride = ahb_planes.planes[i].pixelStride;
+                    ahb_plane_info.row_pitch    = ahb_planes.planes[i].rowStride;
+                    plane_info.emplace_back(std::move(ahb_plane_info));
+                }
+            }
+            else
+            {
+                GFXRECON_LOG_WARNING("AHardwareBuffer_lockPlanes failed: AHardwareBuffer_lock will be used instead");
+            }
+#endif
+
+            if (result != 0)
+            {
+                result =
+                    AHardwareBuffer_lock(hardware_buffer, AHARDWAREBUFFER_USAGE_CPU_READ_OFTEN, -1, nullptr, &data);
+            }
+
+            if (result == 0)
+            {
+                assert(data != nullptr);
+
+                VkAndroidHardwareBufferPropertiesANDROID properties = {
+                    VK_STRUCTURE_TYPE_ANDROID_HARDWARE_BUFFER_PROPERTIES_ANDROID
+                };
+                properties.pNext = nullptr;
+
+                result = device_table->GetAndroidHardwareBufferPropertiesANDROID(
+                    device_unwrapped, hardware_buffer, &properties);
+                if (result == VK_SUCCESS)
+                {
+                    // Only store buffer IDs and reference count if a creation command is written to the capture file.
+                    format::HandleId memory_id = GetUniqueId();
+
+                    HardwareBufferInfo& ahb_info = hardware_buffers_[hardware_buffer];
+                    ahb_info.memory_id           = memory_id;
+                    ahb_info.reference_count     = 0;
+
+                    WriteCreateHardwareBufferCmd(memory_id, hardware_buffer, plane_info);
+                    WriteFillMemoryCmd(memory_id, 0, properties.allocationSize, data);
+
+                    if ((GetMemoryTrackingMode() == CaptureSettings::MemoryTrackingMode::kPageGuard) &&
+                        GetPageGuardTrackAhbMemory())
+                    {
+                        GFXRECON_CHECK_CONVERSION_DATA_LOSS(size_t, properties.allocationSize);
+
+                        util::PageGuardManager* manager = util::PageGuardManager::Get();
+                        assert(manager != nullptr);
+
+                        manager->AddTrackedMemory(
+                            memory_id,
+                            data,
+                            0,
+                            static_cast<size_t>(properties.allocationSize),
+                            util::PageGuardManager::kNullShadowHandle,
+                            false,  // No shadow memory for the imported AHB memory; Track directly with mprotect().
+                            false); // Write watch is not supported for this case.
+                    }
+                }
+                else
+                {
+                    GFXRECON_LOG_ERROR(
+                        "GetAndroidHardwareBufferPropertiesANDROID failed: hardware buffer creation will be "
+                        "omitted from the capture file.");
+                    return false;
+                }
+
+                result = AHardwareBuffer_unlock(hardware_buffer, nullptr);
+                if (result != 0)
+                {
+                    GFXRECON_LOG_ERROR("AHardwareBuffer_unlock failed");
+                }
+            }
+            else
+            {
+                GFXRECON_LOG_ERROR(
+                    "AHardwareBuffer_lock failed: hardware buffer data will be omitted from the capture file");
+                return false;
             }
         }
         else
         {
-            GFXRECON_LOG_WARNING("AHardwareBuffer_lockPlanes failed: AHardwareBuffer_lock will be used instead");
-        }
-#endif
-
-        if (result != 0)
-        {
-            result = AHardwareBuffer_lock(hardware_buffer, AHARDWAREBUFFER_USAGE_CPU_READ_OFTEN, -1, nullptr, &data);
-        }
-
-        if (result == 0)
-        {
-            assert(data != nullptr);
-
+            // The AHB is not CPU-readable, so store only the creation command.
             // Only store buffer IDs and reference count if a creation command is written to the capture file.
             format::HandleId memory_id = GetUniqueId();
 
             HardwareBufferInfo& ahb_info = hardware_buffers_[hardware_buffer];
             ahb_info.memory_id           = memory_id;
-            ahb_info.reference_count     = 1;
-
-            memory_wrapper->hardware_buffer           = hardware_buffer;
-            memory_wrapper->hardware_buffer_memory_id = memory_id;
+            ahb_info.reference_count     = 0;
 
             WriteCreateHardwareBufferCmd(memory_id, hardware_buffer, plane_info);
-            WriteFillMemoryCmd(memory_id, 0, memory_wrapper->allocation_size, data);
-
-            if ((GetMemoryTrackingMode() == CaptureSettings::MemoryTrackingMode::kPageGuard) &&
-                GetPageGuardTrackAhbMemory())
-            {
-                GFXRECON_CHECK_CONVERSION_DATA_LOSS(size_t, memory_wrapper->allocation_size);
-
-                util::PageGuardManager* manager = util::PageGuardManager::Get();
-                assert(manager != nullptr);
-
-                manager->AddTrackedMemory(
-                    memory_id,
-                    data,
-                    0,
-                    static_cast<size_t>(memory_wrapper->allocation_size),
-                    util::PageGuardManager::kNullShadowHandle,
-                    false,  // No shadow memory for the imported AHB memory; Track directly with mprotect().
-                    false); // Write watch is not supported for this case.
-            }
-
-            result = AHardwareBuffer_unlock(hardware_buffer, nullptr);
-            if (result != 0)
-            {
-                GFXRECON_LOG_ERROR("AHardwareBuffer_unlock failed");
-            }
-        }
-        else
-        {
-            GFXRECON_LOG_ERROR(
-                "AHardwareBuffer_lock failed: hardware buffer data will be omitted from the capture file");
         }
     }
 #else
+    GFXRECON_UNREFERENCED_PARAMETER(hardware_buffer);
+#endif
+    return true;
+}
+
+void VulkanCaptureManager::ProcessImportAndroidHardwareBuffer(VkDevice         device,
+                                                              VkDeviceMemory   memory,
+                                                              AHardwareBuffer* hardware_buffer)
+{
+#if defined(VK_USE_PLATFORM_ANDROID_KHR)
+    GFXRECON_UNREFERENCED_PARAMETER(device);
+
+    auto memory_wrapper = reinterpret_cast<DeviceMemoryWrapper*>(memory);
+    assert((memory_wrapper != nullptr) && (hardware_buffer != nullptr));
+
+    bool processing_succeeded = ProcessReferenceToAndroidHardwareBuffer(device, hardware_buffer);
+    if (processing_succeeded)
+    {
+        auto entry = hardware_buffers_.find(hardware_buffer);
+        GFXRECON_ASSERT(entry != hardware_buffers_.end());
+
+        ++entry->second.reference_count;
+
+        memory_wrapper->hardware_buffer           = hardware_buffer;
+        memory_wrapper->hardware_buffer_memory_id = entry->second.memory_id;
+    }
+    else
+    {
+
+        GFXRECON_LOG_ERROR_ONCE(
+            "Importing AHardwareBuffer failed, hardware buffer data will be omitted from the capture file");
+    }
+
+#else
+
     GFXRECON_UNREFERENCED_PARAMETER(device);
     GFXRECON_UNREFERENCED_PARAMETER(memory);
     GFXRECON_UNREFERENCED_PARAMETER(hardware_buffer);
@@ -1914,6 +1988,24 @@ void VulkanCaptureManager::PreProcess_vkQueueSubmit(VkQueue             queue,
     GFXRECON_UNREFERENCED_PARAMETER(pSubmits);
     GFXRECON_UNREFERENCED_PARAMETER(fence);
 
+    QueueSubmitWriteFillMemoryCmd();
+}
+
+void VulkanCaptureManager::PreProcess_vkQueueSubmit2(VkQueue              queue,
+                                                     uint32_t             submitCount,
+                                                     const VkSubmitInfo2* pSubmits,
+                                                     VkFence              fence)
+{
+    GFXRECON_UNREFERENCED_PARAMETER(queue);
+    GFXRECON_UNREFERENCED_PARAMETER(submitCount);
+    GFXRECON_UNREFERENCED_PARAMETER(pSubmits);
+    GFXRECON_UNREFERENCED_PARAMETER(fence);
+
+    QueueSubmitWriteFillMemoryCmd();
+}
+
+void VulkanCaptureManager::QueueSubmitWriteFillMemoryCmd()
+{
     if (GetMemoryTrackingMode() == CaptureSettings::MemoryTrackingMode::kPageGuard)
     {
         util::PageGuardManager* manager = util::PageGuardManager::Get();
@@ -1987,6 +2079,34 @@ void VulkanCaptureManager::PreProcess_vkGetBufferDeviceAddress(VkDevice device,
     }
 }
 
+void VulkanCaptureManager::PreProcess_vkGetBufferOpaqueCaptureAddress(VkDevice                         device,
+                                                                      const VkBufferDeviceAddressInfo* pInfo)
+{
+    auto device_wrapper = reinterpret_cast<DeviceWrapper*>(device);
+    if (!device_wrapper->property_feature_info.feature_bufferDeviceAddressCaptureReplay)
+    {
+        GFXRECON_LOG_ERROR_ONCE(
+            "The application is using vkGetBufferOpaqueCaptureAddress, which requires the "
+            "bufferDeviceAddressCaptureReplay "
+            "feature for accurate capture and replay. The capture device does not support this feature, so replay of "
+            "the captured file may fail.");
+    }
+}
+
+void VulkanCaptureManager::PreProcess_vkGetDeviceMemoryOpaqueCaptureAddress(
+    VkDevice device, const VkDeviceMemoryOpaqueCaptureAddressInfo* pInfo)
+{
+    auto device_wrapper = reinterpret_cast<DeviceWrapper*>(device);
+    if (!device_wrapper->property_feature_info.feature_bufferDeviceAddressCaptureReplay)
+    {
+        GFXRECON_LOG_ERROR_ONCE(
+            "The application is using vkGetDeviceMemoryOpaqueCaptureAddress, which requires the "
+            "bufferDeviceAddressCaptureReplay "
+            "feature for accurate capture and replay. The capture device does not support this feature, so replay of "
+            "the captured file may fail.");
+    }
+}
+
 void VulkanCaptureManager::PreProcess_vkGetAccelerationStructureDeviceAddressKHR(
     VkDevice device, const VkAccelerationStructureDeviceAddressInfoKHR* pInfo)
 {
@@ -2013,6 +2133,24 @@ void VulkanCaptureManager::PreProcess_vkGetRayTracingShaderGroupHandlesKHR(
     }
 }
 
+void VulkanCaptureManager::PreProcess_vkGetAndroidHardwareBufferPropertiesANDROID(
+    VkDevice                                  device,
+    const struct AHardwareBuffer*             hardware_buffer,
+    VkAndroidHardwareBufferPropertiesANDROID* pProperties)
+{
+    GFXRECON_UNREFERENCED_PARAMETER(pProperties);
+#if defined(VK_USE_PLATFORM_ANDROID_KHR)
+    auto device_wrapper = reinterpret_cast<DeviceWrapper*>(device);
+    if (hardware_buffer != nullptr)
+    {
+        ProcessReferenceToAndroidHardwareBuffer(device, const_cast<AHardwareBuffer*>(hardware_buffer));
+    }
+#else
+    GFXRECON_UNREFERENCED_PARAMETER(device);
+    GFXRECON_UNREFERENCED_PARAMETER(hardware_buffer);
+#endif
+}
+
 #if defined(__ANDROID__)
 void VulkanCaptureManager::OverrideGetPhysicalDeviceSurfacePresentModesKHR(uint32_t*         pPresentModeCount,
                                                                            VkPresentModeKHR* pPresentModes)
