@@ -678,6 +678,25 @@ class VulkanCaptureManager : public CaptureManager
         }
     }
 
+    void PostProcess_vkQueueSubmit2(
+        VkResult result, VkQueue queue, uint32_t submitCount, const VkSubmitInfo2* pSubmits, VkFence fence)
+    {
+        if (((GetCaptureMode() & kModeTrack) == kModeTrack) && (result == VK_SUCCESS))
+        {
+            assert((state_tracker_ != nullptr) && ((submitCount == 0) || (pSubmits != nullptr)));
+
+            state_tracker_->TrackCommandBufferSubmissions2(submitCount, pSubmits);
+
+            for (uint32_t i = 0; i < submitCount; ++i)
+            {
+                state_tracker_->TrackSemaphoreInfoSignalState(pSubmits[i].waitSemaphoreInfoCount,
+                                                              pSubmits[i].pWaitSemaphoreInfos,
+                                                              pSubmits[i].signalSemaphoreInfoCount,
+                                                              pSubmits[i].pSignalSemaphoreInfos);
+            }
+        }
+    }
+
     void PostProcess_vkUpdateDescriptorSets(VkDevice,
                                             uint32_t                    descriptorWriteCount,
                                             const VkWriteDescriptorSet* pDescriptorWrites,
@@ -792,6 +811,18 @@ class VulkanCaptureManager : public CaptureManager
         }
     }
 
+    void PostProcess_vkCmdWriteTimestamp2(VkCommandBuffer          commandBuffer,
+                                          VkPipelineStageFlagBits2 pipelineStage,
+                                          VkQueryPool              queryPool,
+                                          uint32_t                 query)
+    {
+        if ((GetCaptureMode() & kModeTrack) == kModeTrack)
+        {
+            assert(state_tracker_ != nullptr);
+            state_tracker_->TrackQueryActivation(commandBuffer, queryPool, query, 0, QueryInfo::kInvalidIndex);
+        }
+    }
+
     void
     PostProcess_vkCmdWriteAccelerationStructuresPropertiesNV(VkCommandBuffer commandBuffer,
                                                              uint32_t        accelerationStructureCount,
@@ -875,6 +906,8 @@ class VulkanCaptureManager : public CaptureManager
 
     void PreProcess_vkQueueSubmit(VkQueue queue, uint32_t submitCount, const VkSubmitInfo* pSubmits, VkFence fence);
 
+    void PreProcess_vkQueueSubmit2(VkQueue queue, uint32_t submitCount, const VkSubmitInfo2* pSubmits, VkFence fence);
+
     void PreProcess_vkCreateDescriptorUpdateTemplate(VkResult                                    result,
                                                      VkDevice                                    device,
                                                      const VkDescriptorUpdateTemplateCreateInfo* pCreateInfo,
@@ -889,6 +922,11 @@ class VulkanCaptureManager : public CaptureManager
 
     void PreProcess_vkGetBufferDeviceAddress(VkDevice device, const VkBufferDeviceAddressInfo* pInfo);
 
+    void PreProcess_vkGetBufferOpaqueCaptureAddress(VkDevice device, const VkBufferDeviceAddressInfo* pInfo);
+
+    void PreProcess_vkGetDeviceMemoryOpaqueCaptureAddress(VkDevice                                      device,
+                                                          const VkDeviceMemoryOpaqueCaptureAddressInfo* pInfo);
+
     void
     PreProcess_vkGetAccelerationStructureDeviceAddressKHR(VkDevice                                           device,
                                                           const VkAccelerationStructureDeviceAddressInfoKHR* pInfo);
@@ -896,6 +934,10 @@ class VulkanCaptureManager : public CaptureManager
     void PreProcess_vkGetRayTracingShaderGroupHandlesKHR(
         VkDevice device, VkPipeline pipeline, uint32_t firstGroup, uint32_t groupCount, size_t dataSize, void* pData);
 
+    void PreProcess_vkGetAndroidHardwareBufferPropertiesANDROID(VkDevice                                  device,
+                                                                const struct AHardwareBuffer*             buffer,
+                                                                VkAndroidHardwareBufferPropertiesANDROID* pProperties);
+
 #if defined(__ANDROID__)
     void OverrideGetPhysicalDeviceSurfacePresentModesKHR(uint32_t* pPresentModeCount, VkPresentModeKHR* pPresentModes);
 #endif
@@ -955,10 +997,13 @@ class VulkanCaptureManager : public CaptureManager
     const VkImportAndroidHardwareBufferInfoANDROID*
     FindAllocateMemoryExtensions(const VkMemoryAllocateInfo* allocate_info);
 
+    bool ProcessReferenceToAndroidHardwareBuffer(VkDevice device, AHardwareBuffer* hardware_buffer);
     void ProcessImportAndroidHardwareBuffer(VkDevice device, VkDeviceMemory memory, AHardwareBuffer* hardware_buffer);
     void ReleaseAndroidHardwareBuffer(AHardwareBuffer* hardware_buffer);
 
   private:
+    void QueueSubmitWriteFillMemoryCmd();
+
     static VulkanCaptureManager*        instance_;
     static LayerTable                   layer_table_;
     std::set<DeviceMemoryWrapper*>      mapped_memory_; // Track mapped memory for unassisted tracking mode.
