@@ -138,7 +138,8 @@ struct DeviceWrapper : public HandleWrapper<VkDevice>
     std::vector<QueueWrapper*> child_queues;
 
     // Physical device property & feature state at device creation
-    graphics::VulkanDevicePropertyFeatureInfo property_feature_info;
+    graphics::VulkanDevicePropertyFeatureInfo              property_feature_info;
+    std::unordered_map<uint32_t, VkDeviceQueueCreateFlags> queue_family_creation_flags;
 };
 
 struct FenceWrapper : public HandleWrapper<VkFence>
@@ -338,9 +339,6 @@ struct DescriptorSetWrapper : public HandleWrapper<VkDescriptorSet>
     // Members for trimming state tracking.
     DeviceWrapper* device{ nullptr };
 
-    const void*        write_pnext{ nullptr };
-    HandleUnwrapMemory write_pnext_memory;
-
     // Map for descriptor binding index to array of descriptor info.
     std::unordered_map<uint32_t, DescriptorInfo> bindings;
 
