@@ -429,34 +429,59 @@ void VulkanStateTracker::TrackCommandBufferSubmissions(uint32_t submit_count, co
                 auto command_wrapper = reinterpret_cast<CommandBufferWrapper*>(command_buffers[cmd]);
                 assert(command_wrapper != nullptr);
 
-                // Apply pending image layouts.
-                for (const auto& layout_entry : command_wrapper->pending_layouts)
-                {
-                    auto image_wrapper = reinterpret_cast<ImageWrapper*>(layout_entry.first);
-                    assert(image_wrapper != nullptr);
+                TrackQuerySubmissions(command_wrapper);
+            }
+        }
+    }
+}
 
-                    image_wrapper->current_layout = layout_entry.second;
-                }
+void VulkanStateTracker::TrackCommandBufferSubmissions2(uint32_t submit_count, const VkSubmitInfo2* submits)
+{
+    if ((submit_count > 0) && (submits != nullptr) && (submits->commandBufferInfoCount > 0))
+    {
+        for (uint32_t submit = 0; submit < submit_count; ++submit)
+        {
+            uint32_t                         command_buffer_count = submits[submit].commandBufferInfoCount;
+            const VkCommandBufferSubmitInfo* command_buffer_infos = submits[submit].pCommandBufferInfos;
 
-                // Apply pending query activations.
-                for (const auto& query_pool_entry : command_wrapper->recorded_queries)
-                {
-                    auto query_pool_wrapper = query_pool_entry.first;
-                    assert(query_pool_wrapper != nullptr);
+            for (uint32_t cmd = 0; cmd < command_buffer_count; ++cmd)
+            {
+                auto command_wrapper = reinterpret_cast<CommandBufferWrapper*>(command_buffer_infos[cmd].commandBuffer);
+                assert(command_wrapper != nullptr);
 
-                    for (const auto& query_entry : query_pool_entry.second)
-                    {
-                        auto& query_info  = query_pool_wrapper->pending_queries[query_entry.first];
-                        query_info.active = query_entry.second.active;
+                TrackQuerySubmissions(command_wrapper);
+            }
+        }
+    }
+}
 
-                        if (query_info.active)
-                        {
-                            query_info.flags              = query_entry.second.flags;
-                            query_info.query_type_index   = query_entry.second.query_type_index;
-                            query_info.queue_family_index = query_entry.second.queue_family_index;
-                        }
-                    }
-                }
+void VulkanStateTracker::TrackQuerySubmissions(CommandBufferWrapper* command_wrapper)
+{
+    // Apply pending image layouts.
+    for (const auto& layout_entry : command_wrapper->pending_layouts)
+    {
+        auto image_wrapper = reinterpret_cast<ImageWrapper*>(layout_entry.first);
+        assert(image_wrapper != nullptr);
+
+        image_wrapper->current_layout = layout_entry.second;
+    }
+
+    // Apply pending query activations.
+    for (const auto& query_pool_entry : command_wrapper->recorded_queries)
+    {
+        auto query_pool_wrapper = query_pool_entry.first;
+        assert(query_pool_wrapper != nullptr);
+
+        for (const auto& query_entry : query_pool_entry.second)
+        {
+            auto& query_info  = query_pool_wrapper->pending_queries[query_entry.first];
+            query_info.active = query_entry.second.active;
+
+            if (query_info.active)
+            {
+                query_info.flags              = query_entry.second.flags;
+                query_info.query_type_index   = query_entry.second.query_type_index;
+                query_info.queue_family_index = query_entry.second.queue_family_index;
             }
         }
     }
@@ -478,13 +503,6 @@ void VulkanStateTracker::TrackUpdateDescriptorSets(uint32_t                    w
             auto                        wrapper = reinterpret_cast<DescriptorSetWrapper*>(write->dstSet);
             assert(wrapper != nullptr);
 
-            wrapper->write_pnext = nullptr;
-            wrapper->write_pnext_memory.Reset();
-            if (write->pNext != nullptr)
-            {
-                wrapper->write_pnext = TrackPNextStruct(write->pNext, &wrapper->write_pnext_memory);
-            }
-
             // Descriptor update rules specify that a write descriptorCount that is greater than the binding's count
             // will result in updates to consecutive bindings, where the next binding is dstBinding+1 and
             // starting from array element 0.  Track the current count, binding, and array element to handle
@@ -498,6 +516,13 @@ void VulkanStateTracker::TrackUpdateDescriptorSets(uint32_t                    w
             {
                 auto& binding = wrapper->bindings[current_binding];
 
+                binding.write_pnext = nullptr;
+                binding.write_pnext_memory.Reset();
+                if (write->pNext != nullptr)
+                {
+                    binding.write_pnext = TrackPNextStruct(write->pNext, &binding.write_pnext_memory);
+                }
+
                 // Update current and write counts for binding's descriptor count. If current count is
                 // greater than the count for the descriptor range defined by dstArrayElement through binding count,
                 // consecutive bindings are being updated.
@@ -1065,6 +1090,35 @@ void VulkanStateTracker::TrackSemaphoreSignalState(uint32_t           wait_count
     }
 }
 
+void VulkanStateTracker::TrackSemaphoreInfoSignalState(uint32_t                     wait_count,
+                                                       const VkSemaphoreSubmitInfo* wait_infos,
+                                                       uint32_t                     signal_count,
+                                                       const VkSemaphoreSubmitInfo* signal_infos)
+{
+    if (((wait_infos != nullptr) && (wait_count > 0)) || ((signal_infos != nullptr) && (signal_count > 0)))
+    {
+        if (wait_infos != nullptr)
+        {
+            for (uint32_t i = 0; i < wait_count; ++i)
+            {
+                auto wrapper = reinterpret_cast<SemaphoreWrapper*>(wait_infos[i].semaphore);
+                assert(wrapper != nullptr);
+                wrapper->signaled = false;
+            }
+        }
+
+        if (signal_infos != nullptr)
+        {
+            for (uint32_t i = 0; i < signal_count; ++i)
+            {
+                auto wrapper = reinterpret_cast<SemaphoreWrapper*>(signal_infos[i].semaphore);
+                assert(wrapper != nullptr);
+                wrapper->signaled = true;
+            }
+        }
+    }
+}
+
 void VulkanStateTracker::TrackAcquireImage(
     uint32_t image_index, VkSwapchainKHR swapchain, VkSemaphore semaphore, VkFence fence, uint32_t deviceMask)
 {
