@@ -51,40 +51,45 @@
 const char kApplicationName[] = "GFXReconstruct Replay";
 const char kCaptureLayer[]    = "VK_LAYER_LUNARG_gfxreconstruct";
 
-const char kHelpShortOption[]                  = "-h";
-const char kHelpLongOption[]                   = "--help";
-const char kVersionOption[]                    = "--version";
-const char kLogLevelArgument[]                 = "--log-level";
-const char kLogFileArgument[]                  = "--log-file";
-const char kLogDebugView[]                     = "--log-debugview";
-const char kNoDebugPopup[]                     = "--no-debug-popup";
-const char kOverrideGpuArgument[]              = "--gpu";
-const char kPausedOption[]                     = "--paused";
-const char kPauseFrameArgument[]               = "--pause-frame";
-const char kSkipFailedAllocationShortOption[]  = "--sfa";
-const char kSkipFailedAllocationLongOption[]   = "--skip-failed-allocations";
-const char kDiscardCachedPsosShortOption[]     = "--dcp";
-const char kDiscardCachedPsosLongOption[]      = "--discard-cached-psos";
-const char kOmitPipelineCacheDataShortOption[] = "--opcd";
-const char kOmitPipelineCacheDataLongOption[]  = "--omit-pipeline-cache-data";
-const char kWsiArgument[]                      = "--wsi";
-const char kSurfaceIndexArgument[]             = "--surface-index";
-const char kMemoryPortabilityShortOption[]     = "-m";
-const char kMemoryPortabilityLongOption[]      = "--memory-translation";
-const char kSyncOption[]                       = "--sync";
-const char kRemoveUnsupportedOption[]          = "--remove-unsupported";
-const char kValidateOption[]                   = "--validate";
-const char kDebugDeviceLostOption[]            = "--debug-device-lost";
-const char kCreateDummyAllocationsOption[]     = "--create-dummy-allocations";
-const char kDeniedMessages[]                   = "--denied-messages";
-const char kAllowedMessages[]                  = "--allowed-messages";
-const char kShaderReplaceArgument[]            = "--replace-shaders";
-const char kScreenshotAllOption[]              = "--screenshot-all";
-const char kScreenshotRangeArgument[]          = "--screenshots";
-const char kScreenshotFormatArgument[]         = "--screenshot-format";
-const char kScreenshotDirArgument[]            = "--screenshot-dir";
-const char kScreenshotFilePrefixArgument[]     = "--screenshot-prefix";
-const char kOutput[]                           = "--output";
+const char kHelpShortOption[]                    = "-h";
+const char kHelpLongOption[]                     = "--help";
+const char kVersionOption[]                      = "--version";
+const char kLogLevelArgument[]                   = "--log-level";
+const char kLogFileArgument[]                    = "--log-file";
+const char kLogDebugView[]                       = "--log-debugview";
+const char kNoDebugPopup[]                       = "--no-debug-popup";
+const char kOverrideGpuArgument[]                = "--gpu";
+const char kPausedOption[]                       = "--paused";
+const char kPauseFrameArgument[]                 = "--pause-frame";
+const char kSkipFailedAllocationShortOption[]    = "--sfa";
+const char kSkipFailedAllocationLongOption[]     = "--skip-failed-allocations";
+const char kDiscardCachedPsosShortOption[]       = "--dcp";
+const char kDiscardCachedPsosLongOption[]        = "--discard-cached-psos";
+const char kOmitPipelineCacheDataShortOption[]   = "--opcd";
+const char kOmitPipelineCacheDataLongOption[]    = "--omit-pipeline-cache-data";
+const char kWsiArgument[]                        = "--wsi";
+const char kSurfaceIndexArgument[]               = "--surface-index";
+const char kMemoryPortabilityShortOption[]       = "-m";
+const char kMemoryPortabilityLongOption[]        = "--memory-translation";
+const char kSyncOption[]                         = "--sync";
+const char kRemoveUnsupportedOption[]            = "--remove-unsupported";
+const char kValidateOption[]                     = "--validate";
+const char kDebugDeviceLostOption[]              = "--debug-device-lost";
+const char kCreateDummyAllocationsOption[]       = "--create-dummy-allocations";
+const char kOmitNullHardwareBuffersLongOption[]  = "--omit-null-hardware-buffers";
+const char kOmitNullHardwareBuffersShortOption[] = "--onhb";
+const char kDeniedMessages[]                     = "--denied-messages";
+const char kAllowedMessages[]                    = "--allowed-messages";
+const char kShaderReplaceArgument[]              = "--replace-shaders";
+const char kScreenshotAllOption[]                = "--screenshot-all";
+const char kScreenshotRangeArgument[]            = "--screenshots";
+const char kScreenshotFormatArgument[]           = "--screenshot-format";
+const char kScreenshotDirArgument[]              = "--screenshot-dir";
+const char kScreenshotFilePrefixArgument[]       = "--screenshot-prefix";
+const char kOutput[]                             = "--output";
+const char kMeasurementRangeArgument[]           = "--measurement-frame-range";
+const char kQuitAfterMeasurementRangeOption[]    = "--quit-after-measurement-range";
+const char kFlushMeasurementRangeOption[]        = "--flush-measurement-range";
 #if defined(WIN32)
 const char kApiFamilyOption[] = "--api";
 #endif
@@ -469,6 +474,91 @@ GetScreenshotRanges(const gfxrecon::util::ArgumentParser& arg_parser)
     return ranges;
 }
 
+static bool
+GetMeasurementFrameRange(const gfxrecon::util::ArgumentParser& arg_parser, uint32_t& start_frame, uint32_t& end_frame)
+{
+    start_frame = 1;
+    end_frame   = std::numeric_limits<uint32_t>::max();
+
+    const auto& value = arg_parser.GetArgumentValue(kMeasurementRangeArgument);
+    if (!value.empty())
+    {
+        std::string range = value;
+
+        if (std::count(range.begin(), range.end(), '-') != 1)
+        {
+            GFXRECON_LOG_WARNING(
+                "Ignoring invalid measurement frame range \"%s\". Must have format: <start_frame>-<end_frame>",
+                range.c_str());
+            return false;
+        }
+
+        // Remove whitespace.
+        range.erase(std::remove_if(range.begin(), range.end(), ::isspace), range.end());
+
+        // Split string on '-' delimiter.
+        bool                     invalid = false;
+        std::vector<std::string> values;
+        std::istringstream       range_input;
+        range_input.str(range);
+
+        for (std::string token; std::getline(range_input, token, '-');)
+        {
+            if (token.empty())
+            {
+                break;
+            }
+
+            // Check that the range string only contains numbers.
+            size_t count = std::count_if(token.begin(), token.end(), ::isdigit);
+            if (count == token.length())
+            {
+                values.push_back(token);
+            }
+            else
+            {
+                GFXRECON_LOG_WARNING(
+                    "Ignoring invalid measurement frame range \"%s\", which contains non-numeric values",
+                    range.c_str());
+                invalid = true;
+                break;
+            }
+        }
+
+        if (values.size() < 2)
+        {
+            GFXRECON_LOG_WARNING("Ignoring invalid measurement frame range \"%s\", does not have two values.",
+                                 range.c_str());
+
+            invalid = true;
+        }
+
+        if (!invalid)
+        {
+            uint32_t start_frame_arg = std::stoi(values[0]);
+            uint32_t end_frame_arg   = std::stoi(values[1]);
+
+            if (start_frame_arg >= end_frame_arg)
+            {
+                GFXRECON_LOG_WARNING("Ignoring invalid measurement frame range \"%s\", where first frame is "
+                                     "greater than or equal to the last frame",
+                                     range.c_str());
+
+                return false;
+            }
+
+            start_frame = start_frame_arg;
+            end_frame   = end_frame_arg;
+            return true;
+        }
+        else
+        {
+            GFXRECON_LOG_WARNING("Ignoring invalid measurement frame range \"%s\".", range.c_str());
+        }
+    }
+
+    return false;
+}
 static gfxrecon::decode::CreateResourceAllocator
 GetCreateResourceAllocatorFunc(const gfxrecon::util::ArgumentParser&           arg_parser,
                                const std::string&                              filename,
@@ -581,6 +671,22 @@ static void GetReplayOptions(gfxrecon::decode::ReplayOptions& options, const gfx
     {
         options.create_dummy_allocations = true;
     }
+
+    if (arg_parser.IsOptionSet(kOmitNullHardwareBuffersLongOption) ||
+        arg_parser.IsOptionSet(kOmitNullHardwareBuffersShortOption))
+    {
+        options.omit_null_hardware_buffers = true;
+    }
+
+    if (arg_parser.IsOptionSet(kQuitAfterMeasurementRangeOption))
+    {
+        options.quit_after_measurement_frame_range = true;
+    }
+
+    if (arg_parser.IsOptionSet(kFlushMeasurementRangeOption))
+    {
+        options.flush_measurement_frame_range = true;
+    }
 }
 
 static gfxrecon::decode::VulkanReplayOptions
@@ -628,6 +734,15 @@ GetVulkanReplayOptions(const gfxrecon::util::ArgumentParser&           arg_parse
     replay_options.screenshot_format      = GetScreenshotFormat(arg_parser);
     replay_options.screenshot_dir         = GetScreenshotDir(arg_parser);
     replay_options.screenshot_file_prefix = arg_parser.GetArgumentValue(kScreenshotFilePrefixArgument);
+    if (arg_parser.IsOptionSet(kQuitAfterMeasurementRangeOption))
+    {
+        replay_options.quit_after_measurement_frame_range = true;
+    }
+
+    if (arg_parser.IsOptionSet(kFlushMeasurementRangeOption))
+    {
+        replay_options.flush_measurement_frame_range = true;
+    }
 
     std::string surface_index = arg_parser.GetArgumentValue(kSurfaceIndexArgument);
     if (!surface_index.empty())
